// Copyright (c)1995 Ray Dream, Inc. All Rights Reserved.
/*$Id: DefSFac.cpp 1.2 1997/05/05 22:47:02 damien Exp $*/

#ifndef __DEFSFAC__
#include "DEFSFac.h"
#endif

#ifndef __COMDEFS__
#include "COMDEFS.h"
#endif

#ifndef __DEFSDLL__
#include "DEFSdll.h"
#endif


// ***** ClassFactory *****
  
DeformerClassFactory::DeformerClassFactory() {
  m_cRef=0L;  // just created so no reference used
  return;
  }


DeformerClassFactory::~DeformerClassFactory() {
  return;
  }

// IUnknown methods of DeformerClassFactory                                                
STDMETHODIMP DeformerClassFactory::QueryInterface(REFIID riid, LPVOID* ppv) {
  *ppv=NULL;

  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory))
    *ppv=(LPVOID)this;

  if (*ppv!=NULL) {
    ((LPUNKNOWN)*ppv)->AddRef(); // Add reference if we give a pointer
    return NOERROR;
    }
  else {
    return ResultFromScode(E_NOINTERFACE);
    }
  }


STDMETHODIMP_(ULONG) DeformerClassFactory::AddRef() {
  return ++m_cRef;
  }


STDMETHODIMP_(ULONG) DeformerClassFactory::Release() {
  ULONG   UnreleaseRef;

  UnreleaseRef=--m_cRef;

  if (m_cRef == 0)
    delete this; // No reference left, so delete the DeformerClassFactory

  return UnreleaseRef;
  }

/*
 * DeformerClassFactory::CreateInstance
 *
 * Parameters:
 *  pUnkOuter       LPUNKNOWN to the controlling IUnknown if we are
 *                  being used in an aggregation.
 *  riid            REFIID identifying the interface the caller
 *                  desires to have for the new object.
 *  ppvObj          LPVOID FAR* in which to store the desired
 *                  interface pointer for the new object.
 *
 * Return Value:
 *  HRESULT         NOERROR if successful, otherwise E_NOINTERFACE
 *                  if we cannot support the requested interface.
 */

STDMETHODIMP DeformerClassFactory::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, LPVOID FAR* ppvObj) {
  Deformer*      pObj = NULL;
  HRESULT       hr;

  *ppvObj = NULL;
  hr = ResultFromScode(E_OUTOFMEMORY);

  //Verify that a controlling unknown asks for IUnknown
  if (pUnkOuter && !IsEqualIID(riid, IID_IUnknown))
    return ResultFromScode(E_NOINTERFACE);

  //Create the object passing function to notify on destruction.
  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_I3DExDeformer2))
    pObj = new Deformer;
  else
    return ResultFromScode(E_NOINTERFACE);

  hr=pObj->QueryInterface(IID_I3DExDeformer2, ppvObj);

  //Kill the object if initial creation failed.
  if (FAILED(hr))
    delete pObj;
  else
    global_count_Obj++;

  return hr;
  }


/*
 * DeformerClassFactory::LockServer
 *
 * Purpose:
 *  Increments or decrements the lock count of the DLL.  If the
 *  lock count goes to zero and there are no objects, the DLL
 *  is allowed to unload.  See DllCanUnloadNow in "Deformerdll.cpp".
 *
 * Parameters:
 *  fLock           BOOL specifying whether to increment or
 *                  decrement the lock count.
 *
 * Return Value:
 *  HRESULT         NOERROR always.
 */

STDMETHODIMP DeformerClassFactory::LockServer(BOOL fLock) {
  if (fLock)
      global_count_Lock++;
  else
      global_count_Lock--;

  return NOERROR;
  }

